;********* AVRISP Bootloader for AT90S2313, ATtiny2313 by RA4NAL *********

;     ATtiny2313
;=========================================================================
; :
; source: www.atmel.ru/Disks/AVR%20Technical%20Library/index.html
; Appnote AVR109: Self Programming AVR  --> avr109.pdf, avr109.zip
; Appnote AVR910: In-System Programming --> avr910.pdf, avr910.zip

; source: www.klaus-leidinger.de/mp/Mikrocontroller/AVR-Prog/AVR-AN910-V3.1-orig.zip
; avr910_31.asm, Author: wubblick@yahoo.com

; source: www.klaus-leidinger.de/mp/Mikrocontroller/AVR-Prog/AVR-Programmer.html
; avr910_2313_V37.asm. Author: klaus@mikrocontroller-projekte.de
;=========================================================================

;**** includes ****
        .NOLIST
        .include "tn2313def.inc"
        .LIST

;**** Constant declarations Data Rate ****

   .equ XTAL = 3686;   ; XTAL frequency, KHz
   .equ BAUD = 19200   ; Data rate, bauds

   .equ NB = ((XTAL*10000)/(16*BAUD)-5)/10

   .equ RAMSTART = 0x60  ; first SRAM Adress of 2313
   .equ PSIZE    = 0x20  ; Page size for m8535
   .equ BUFSIZE  = 0x40  ; 64 bytes internal RAM for Buffer

;***************************************************************************
;* PORTS
;* Ports Definitions
;***************************************************************************
   .equ MISO  = PORTB6  ; MISO  pin of the target (input)
   .equ MOSI  = PORTB5  ; MOSI  pin of the target (output)
   .equ RESET = PORTB4  ; RESET pin of the target (output)
   .equ SCK   = PORTB7  ; SCK   pin of the target (output)
   .equ RXD   = PORTD0  ; UART RXD line
   .equ TXD   = PORTD1  ; UART TXD line

;***************************************************************************
;* MACROS
;***************************************************************************
.macro table             ; load Z pointer
     ldi   ZL,low(@0*2)  ; low
     ldi   ZH,high(@0*2) ; high
.endm

;***************************************************************************
;*
;* Global Register Variables
;*
;***************************************************************************

;r0 used with lpm instruction
.def cmd1      = r1  ; Universal commands params
.def cmd2      = r2  ; ..
.def cmd3      = r3  ; ..
.def pol_al    = r4  ; Polling address low
.def pol_ah    = r5  ; Polling address high
.def Bcnt1     = r6  ; Block Mode Counter1
.def Bcnt2     = r7  ; Block Mode Counter2
.def Bcnt3     = r8  ; Block Mode Counter2
.def B_Flag    = r9  ; Flag for ws_del Routine
.def B_Mode    = r10 ; Marks Block Mode commands (See Note 16)
.def Memtype   = r11 ; Flag for Memorytype for Block Commands (See Note 16)
.def Pagewords = r12 ; Size of Controller page to program in Page Mode (Words!)
.def PollcodeF = r13 ; Code for Polling Flash

.def temp1     = r16 ; Temporary register 1
.def temp2     = r17 ; Temporary register 2
.def temp3     = r18 ; Temporary register 3
.def s_data    = r19 ; SPI data
.def u_data    = r20 ; UART data
.def device    = r21 ; Device code
.def rd_s_data = r23 ; Read data on SPI
.def pol_cmd   = r24 ; Polling command
.def p_data    = r25 ; Polling data
;r26,r27 used as X register
.def addrl     = r28 ; (YL) Low order byte of address
.def addrh     = r29 ; (YH) High order byte of address
;r30,r31 used as Z register

        .org  $0000

        rjmp  INIT   ; Reset Handle
        rjmp  $0000
        rjmp  $0000
        rjmp  $0000
        rjmp  $0000
        rjmp  $0000
        rjmp  $0000
        rjmp  $0000
        rjmp  $0000
        rjmp  $0000
        rjmp  $0000
        rjmp  $0000
        rjmp  $0000
        rjmp  $0000
        rjmp  $0000
        rjmp  $0000
        rjmp  $0000
        rjmp  $0000
        rjmp  $0000

;***************************************************************************
;* FUNCTION
;* u_init
;*
;* DESCRIPTION
;* Initialize UART.
;***************************************************************************
u_init:
        ldi   temp1,00
        out   UBRRH,temp1
        ldi   temp1,NB   ; set baud rate
        out   UBRRL,temp1
        ldi   temp1,24   ; initialize UART for TX and RX
        out   UCSRB,temp1
        ldi   temp1,6
        out   UCSRC,temp1
        ret
;***************************************************************************
;* FUNCTION
;* getc
;*
;* DESCRIPTION
;* Wait for start bit and receive a character on the UART Rx line.
;***************************************************************************
getc:
        sbis  UCSRA,RXC  ; wait until a character has been received
        rjmp  getc
        in    u_data,UDR ; Read byte from the UART
        ret

;***************************************************************************
;* FUNCTION
;* putc
;*
;* DESCRIPTION
;* Send a character on the UART Tx line.
;***************************************************************************
putc:
        sbis  UCSRA,UDRE ; test for TX register empty
        rjmp  putc       ; loop until TX empty
        out   UDR,u_data ; send the byte
        ret

;***************************************************************************
;* FUNCTION
;* put_string
;*
;* DESCRIPTION
;* Send Z - pointed null-terminated string on the UART Tx line.
;***************************************************************************
put_string:
        lpm
        tst   r0
        breq  ps_ret     ; check for end of string (0x00)
        mov   u_data,r0
        rcall putc       ; putc(char)
        adiw  ZL,1       ; next char
        rjmp  put_string
ps_ret: ret

;***************************************************************************
;* FUNCTION
;* put_table
;*
;* DESCRIPTION
;* Send Z - pointed table on the UART Tx line.
;***************************************************************************
put_table:
        lpm
        tst   r0
        breq  pt_ret   ; check for end of table (0x00)
        mov   u_data,r0
        rcall putc   ; putc(Byte)
        adiw  ZL,2   ; skip MSB
        rjmp  put_table
pt_ret: ret

;***************************************************************************
;* FUNCTION
;* bel_table
;*
;* DESCRIPTION
;* C=0 if device belongs to table.
;***************************************************************************
bel_table:
        lpm            ; read table
        tst   r0       ; check for end of table
        breq  c1_ret
        cp    device,r0
        breq  c0_ret   ; C=0
        adiw  ZL,2     ; skip MSB
        rjmp  bel_table
c1_ret: sec            ; C=1
c0_ret: ret  

;***************************************************************************
;* FUNCTION
;* set_pagesize
;*
;* DESCRIPTION
;* sets programming Page size for selected Device.
;***************************************************************************
set_pagesize:
        lpm                    ; read table
        tst   r0               ; check for end of table
        breq  spa_end          ; no Pagesize to set
        cp    device,r0
        breq  spa_set          ; C=0
        adiw  ZL,2             ; skip MSB
        rjmp  set_pagesize
spa_set:
        adiw  ZL,1             ; Point to high Byte of Word
        lpm                    ; get Pagesize to R0
        mov   Pagewords,r0
        mov   Bcnt3,Pagewords  ; initiate Counter
spa_end:
        ret

;***************************************************************************
;* FUNCTION
;* set_pollcode
;*
;* DESCRIPTION
;* sets Code for Polling Flash for selected Device.
;***************************************************************************
set_pollcode:
        lpm                  ; read table
        tst   r0             ; check for end of table
        breq  spo_end        ; no Pollcode to set
        cp    device,r0
        breq  spo_set        ; C=0
        adiw  ZL,2           ; skip MSB
        rjmp  set_pollcode
spo_set:
        adiw  ZL,1           ; Point to high Byte of Word
        lpm                  ; get to R0
        mov   PollcodeF,r0   ; Set Pollcode for Flash Rom
spo_end:
        ret

;***************************************************************************
;* FUNCTION
;* delay
;*
;* DESCRIPTION
;*  Make delay 1mS (x temp1).
;***************************************************************************
delay:
        ldi   temp2,40
dl2:    ldi   temp3,(XTAL/120)
dl1:    dec   temp3
        brne  dl1
        dec   temp2
        brne  dl2
        dec   temp1
        brne  delay
        ret

;***************************************************************************
;* FUNCTION
;*  spi123
;*
;* DESCRIPTION
;*  Write bytes 1 to 3 on the SPI. Byte 1 must be loadet into s_data
;*  Byte 2 ist addrh, Byte 3 ist addrl
;***************************************************************************
spi123:
        rcall wrser   ; wrser(s_data) SPI write (byte 1)
        mov   s_data,addrh
        rcall wrser   ; wrser(addrh)  SPI write (byte 2)
        mov   s_data,addrl
        rcall wrser   ; wrser(addrl)  SPI write (byte 3)
        ret

;***************************************************************************
;* FUNCTION
;*  w1234 (for Code simplification,)
;*
;* DESCRIPTION
;*  Write SPI bytes 1 to 4.
;            Byte 1 must be loadet into cmd1
;            Byte 2 must be loadet into cmd2
;            Byte 3 must be loadet into cmd3
;            Byte 4 must be loadet into u_data
;***************************************************************************
w1234:
        mov   s_data,cmd1
        rcall wrser
        mov   s_data,cmd2
        rcall wrser
        mov   s_data,cmd3
        rcall wrser
        mov   s_data,u_data  ; get Byte 4 from serial
        rcall wrser
        ret

;***************************************************************************
;* FUNCTION
;*  w123r4 (for Code simplification)
;*
;* DESCRIPTION
;*  Write SPI bytes 1 to 3, read Byte 4 to serial.
;            Byte 1 must be loadet into cmd1
;            Byte 2 must be loadet into cmd2
;            Byte 3 must be loadet into cmd3
;***************************************************************************
w123r4:
        mov   s_data,cmd1
        rcall wrser
        mov   s_data,cmd2
        rcall wrser
        mov   s_data,cmd3
        rcall wrser
        rcall rdser
        mov   u_data,s_data  ; put Byte 4 to serial
        rcall putc
        ret

;***************************************************************************
;* FUNCTION
;* rdser, wrser
;*
;* DESCRIPTION
;*  Write and read bytes on the SPI.
;***************************************************************************
rdser:
        clr   s_data
wrser: 
        ldi   temp1,8   ; load bit counter
        ldi   rd_s_data,0
wrs0:
        rol   s_data
        brcc  wrs1
        sbi   PORTB,MOSI   ; MOSI = 1
        rjmp  wrs2
wrs1:
        cbi   PORTB,MOSI   ; MOSI = 0
wrs2:
        lsl   rd_s_data
        sbic  PINB,MISO    ; skip if MISO cleared
        ori   rd_s_data,1
        sbi   PORTB,SCK    ; pulse SCK
        nop
        nop
        nop
        nop
        cbi   PORTB,SCK
        dec   temp1    ; advance bit counter
        brne  wrs0   ; loop
        mov   s_data,rd_s_data
        ret
;***************************************************************************
;* FUNCTION
;* read_send_progmem
;*
;* DESCRIPTION
;*  Read one adress (2 Byte) from Program Memory and send it through UART
;***************************************************************************
read_send_progmem:
        ldi   s_data,0x20  ; read low Byte
        rcall wrser        ; wrser(0x28)  SPI write (byte 1)
        mov   s_data,addrh
        rcall wrser        ; wrser(addrh) SPI write (byte 2)
        mov   s_data,addrl
        rcall wrser        ; wrser(addrl) SPI write (byte 3)
        rcall rdser        ;              SPI read  (byte 4)
        mov   u_data,s_data
        rcall putc         ; send data
        ldi   s_data,0x28  ; read High Byte
        rcall wrser        ; wrser(0x20)  SPI write (byte 1)
        mov   s_data,addrh
        rcall wrser        ; wrser(addrh) SPI write (byte 2)
        mov   s_data,addrl
        rcall wrser        ; wrser(addrl) SPI write (byte 3)
        rcall rdser        ;              SPI read  (byte 4)
        mov   u_data,s_data
        rcall putc         ; send data
        adiw  addrl,1      ; Auto increment address
        ret

;***************************************************************************
;* FUNCTION
;* read_send_datamem
;*
;* DESCRIPTION
;*  Read one Byte from Data Memory (eeprom) and send it through UART
;***************************************************************************
read_send_datamem:         ; Subroutine to read one eeprom Address
        ldi   s_data,0xa0
        rcall wrser        ; wrser(0xa0)   SPI write (byte 1)
        mov   s_data,addrh
        rcall wrser        ; wrser(addrh)  SPI write (byte 2)
        mov   s_data,addrl
        rcall wrser        ; wrser(addrl)  SPI write (byte 3)
        rcall rdser        ;               SPI read  (byte 4)
        mov u_data,s_data
        rcall putc         ; send data
        adiw  addrl,1      ; Auto increment address
        ret

;***************************************************************************
;* FUNCTION
;* eeprom_write
;*
;* DESCRIPTION
;*  Write u_data to Data Memory (eeprom)
;***************************************************************************
eeprom_write:
        ldi   s_data,0xc0
        rcall wrser        ; wrser(0xc0)   SPI write (byte 1)
        mov   s_data,addrh
        rcall wrser        ; wrser(addrh)  SPI write (byte 2)
        mov   s_data,addrl
        rcall wrser        ; wrser(addrl)  SPI write (byte 3)
        mov   s_data,u_data
        rcall wrser        ; wrser(u_data) SPI write (byte 4)
        ldi   temp1,10     ; delay 10mS
        rcall delay
        adiw  addrl,1      ; Auto increment address
        ret

;***************************************************************************
;*
;* INIT
;*
;* DESCRIPTION
;* Initialization
;*
;***************************************************************************
INIT:
        ldi   temp1,RAMEND
        out   SPL,temp1       ; Locate stack
        ldi   temp1,PSIZE     ; Pagesize
        mov   Pagewords,temp1
        mov   Bcnt3,Pagewords ; set counter for Pagesize
        ldi   device,0x40     ; M8535 as default
        clr   B_Mode          ; Flag for Block Modes
        clr   B_Flag          ; Flag for eNhanced Block write
        clr   temp2           ; PD = 0, IN
        out   PORTD,temp2
        out   TCCR1A,temp2    ; . F  PB3
        out   TCCR1C,temp2
        ldi   temp2,0x07      ; 0000.0111 MISO,MOSI,SCK,RESET,XT1 = 0
        out   PORTB,temp2
        ldi   temp1,10        ; delay 10mS
        rcall delay
        sbic  PINB,0          ;  SB1
        rjmp  VccON           ; 
        ldi   temp2,0x00      ; PD - IN
        out   DDRD,temp2
        ldi   temp2,0xb0      ; 10110000 PB7,5,4 = -OUT
        out   DDRB,temp2
        rcall u_init          ; Initialize UART

;***************************************************************************
;*
;* PROGRAM
;* waitcmd -&gt; main
;*
;* DESCRIPTION
;* Wait for and execute commands.
;*
;***************************************************************************
waitcmd:
        rcall getc            ; while (getc() == ESC) {};
        cpi   u_data,0x1b
        breq  waitcmd
       
;====== 'T' - Device Type ==================================================
        cpi   u_data,'T'      ; 'T' Device type
        brne  w0
        rcall getc
        mov   device,u_data   ; get device type
     table Dev_M              ; prepare to set Pagesize
        rcall set_pagesize    ; If device has Page Mode support, set Pagesize
        ldi   temp1,0xFF
        mov   PollcodeF,temp1 ; preset PollcodeF with 0xFF
     table Dev_S              ; prepare to set Polling Code
        rcall set_pollcode    ; If no Page Size support, set code for Polling
        rjmp  put_ret

;====== 'S' - Return Software Identifier ===================================
w0:
        cpi   u_data,'S'  ; 'S' Return software identifier
        brne  w1
     table ID_Str
        rcall put_string  ; put string "AVR ISP"
        rjmp  waitcmd
       
;====== 'V' - Return Software Version ======================================
w1:
        cpi   u_data,'V'  ; 'V' Return software version
        brne  w2
        table SW_Ver
        rcall put_string  ; put software version
        rjmp  waitcmd

;====== 'v' - Return Hardware Version ======================================
w2:
        cpi   u_data,'v'  ; 'v' Return hardware version
        brne  w3
     table HW_Ver
        rcall put_string  ; put hardware version
        rjmp  waitcmd

;====== 't' - Show Supported Devices =======================================
w3:
        cpi   u_data,'t'  ; 't' Show supported devices
        brne  w4
     table Dev_S
        rcall put_table   ; put supported devices codes
     table Dev_M
        rcall put_table   ; put supported devices codes
        ldi   u_data,0x00 ; putc(0x00) - end of device list
        rcall putc
        rjmp  waitcmd
       
;====== 'p' - Return Programmer Type =======================================
w4:
        cpi   u_data,'p'  ; 'p' Return programmer type
        brne  w5
        ldi   u_data,'S'  ; putc('S') - serial programmer
        rcall putc
        rjmp  waitcmd

;====== 'a' - Return autoincrement address support =========================
w5:
        cpi   u_data,'a'  ; 'a' Return address auto increment
        brne  w53
        ldi   u_data,'Y'  ; putc('Y') - supports autoinc
        rcall putc
        rjmp  waitcmd

;====== 'b' - Return Block write Mode support (Note 17) ========================
w53:
        cpi   u_data,'b'  ; 'b' Return enhanced mode Support
        brne  w6
        ldi   u_data,'Y'  ; putc('Y') - supports enhanced Mode
        rcall putc
        ldi   u_data,high(BUFSIZE); putc((BUFSIZE&gt;&gt;8) &amp; 0xff);
        rcall putc
        ldi   u_data,low(BUFSIZE) ; putc(BUFSIZE&amp;0xff);
        rcall putc
        rjmp  waitcmd
       
;====== 'x' - Set LED   =====================================
w6:
        cpi   u_data,'x'  ; 'x' Set LED (LED off or green)
        brne  w61
        rcall getc   ; get parameter
        rjmp  put_ret
       
;====== 'y' - Clear LED   ===================================
w61:
        cpi   u_data,'y'  ; 'y' Clear LED (LED on or red)
        brne  w7
        rcall getc   ; get parameter
        rjmp  put_ret

;===========================================================================
; We require that the device code be selected before any of the other commands
w7:
     table Dev_S     ; load pointer
        rcall bel_table
        brcc  w71    ; device belongs to table
     table Dev_M 
        rcall bel_table
        brcc  w71    ; device belongs to table
        rjmp  put_err ; not match, goto put_err();

;====== 'P' - Enter Programming Mode =======================================
w71:
        cpi   u_data,'P'   ; 'P' Enter programming mode
        breq  w70
        rjmp  w8
w70:
        ldi   temp2,0x00   ; PD - IN
        out   DDRD,temp2
        ldi   temp2,0xb0   ; 10110000 PB7,5,4 = -OUT
        out   DDRB,temp2
        cbi   PORTB,SCK    ; clear SCK
        sbi   PORTB,RESET  ; set RESET passive
        ldi   temp1,50     ; delay 50mS;
        rcall delay
        cbi   PORTB,RESET  ; set RESET active
        ldi   temp1,50     ; delay 50mS;
        rcall delay 
        ldi   s_data,0xac
        rcall wrser        ; wrser(0xac) SPI write (byte 1)
        ldi   s_data,0x53
        rcall wrser        ; wrser(0x53) SPI write (byte 2)
; SPI Synchronization
        cpi   device,0x20  ; if ( (device &gt;= 0x20) &amp;&amp; (device &lt;= 0x7F) )
        brlo  s2
        ldi   temp3,32     ; count = 32;
s1:     rcall rdser        ; SPI read  (byte 3)
        cpi   s_data,0x53  ; if (rdser == 0x53)
        breq  s3           ; break
        ldi   s_data,0x00
        rcall wrser        ; wrser(0x00) SPI write (byte 4)
        sbi   PORTB,SCK    ; pulse SCK
        nop
        nop
        nop
        nop
        cbi   PORTB,SCK
        ldi   s_data,0xac
        rcall wrser         ; wrser(0xac) SPI write (byte 1)
        ldi   s_data,0x53
        rcall wrser         ; wrser(0x53) SPI write (byte 2)
        dec   temp3         ; count-1
        brne  s1            ; loop
        rjmp  s3            ; else
s2:     ldi   s_data,0x00
        rcall wrser         ; wrser(0x00) SPI write (byte 3)
s3:     ldi   s_data,0x00
        rcall wrser         ; wrser(0x00) SPI write (byte 4)
s4:     ldi   temp1,4       ; delay 4mS;
        rcall delay
        rjmp  put_ret

;====== 'c' - Write Program Memory, Low Byte ===============================
w8:
        cpi   u_data,'c'    ; 'c' Write program memory, low byte
        brne  w9
        rcall getc          ; get data byte
w8b:    ldi   s_data,0x40
        mov   pol_cmd,s_data; save command for polling
        rcall wrser         ; wrser(0x40)   SPI write (byte 1)
        mov   s_data,addrh
        rcall wrser         ; wrser(addrh)  SPI write (byte 2)
        mov   s_data,addrl
        rcall wrser         ; wrser(addrl)  SPI write (byte 3)
        mov   s_data,u_data
        rcall wrser         ; wrser(u_data) SPI write (byte 4)
        mov   p_data,u_data ; save data for polling
        mov   pol_al,addrl  ; save address for polling
        mov   pol_ah,addrh
        rjmp  wait_S        ; write FLASH delay

;====== 'C' - Write Program Memory, High Byte ==============================
w9:
        cpi   u_data,'C'    ; 'C' Write program memory, high byte
        brne  w92
        rcall getc          ; get data byte
        ldi   s_data,0x48
        mov   pol_cmd,s_data; save command for polling
        rcall wrser         ; wrser(0x48)   SPI write (byte 1)
        mov   s_data,addrh
        rcall wrser         ; wrser(addrh)  SPI write (byte 2)
        mov   s_data,addrl
        rcall wrser         ; wrser(addrl)  SPI write (byte 3)
        mov   s_data,u_data ; wrser(u_data) SPI write (byte 4)
        rcall wrser
        mov   p_data,u_data ; save data for polling
        mov   pol_al,addrl  ; save address for polling
        mov   pol_ah,addrh
        adiw  addrl,1       ; Auto increment address
        rjmp  wait_S        ; write FLASH delay
       
;====== 'B' Block Write Memory =============================================
w92:
        cpi   u_data,'B'    ; 'B' Block Write Program Memory
        breq  w92a
        rjmp  w10
w92a:
        rcall getc          ; get count High Byte
        tst   u_data
        breq  w92b
        ldi   u_data,1
        rjmp  put_err
w92b:
        rcall getc          ; get count Low Byte
        cpi   u_data,BUFSIZE+1 ; check maximum count
        brlo  w92c
        ldi   u_data,2
        rjmp  put_err
w92c:
        mov   Bcnt1,u_data    ; ignore BUFSIZE high Byte (must be 0 here)
        mov   Bcnt2,u_data
        rcall getc            ; get Memory type to write
        mov   Memtype,u_data  ; Flag for Memtype
        clr   XH
        ldi   XL,RAMSTART     ; set X pointer to SRAM begin
w93:
        rcall getc            ; get data until Bcnt1 is reached
        st    X+,u_data       ; store data to SRAM
        dec   Bcnt1
        brne  w93
        clr   XH
        ldi   XL,RAMSTART     ; set X pointer to SRAM begin
        ldi   temp3,1
        mov   B_Mode,temp3    ; B_Mode != 0
        mov   u_data,Memtype  ; restore Memtype
        cpi   u_data,'F'      ; 'F' Flash Memory ist to write
        breq  w94a
        cpi   u_data,'E'      ; 'E' eeprom Memory ist to write
        breq  w94             ; Entry Point for Data Memory Block write
        ldi   u_data,3
        rjmp  put_err
w94:                          ; write Data (EEPROM) Memory
        ld    u_data,X+
        rcall eeprom_write
        dec   Bcnt2
        brne  w94
        rjmp  put_ret

;Entry Point for Flash Block write
w94a:
     table Dev_M 
        rcall bel_table
        brcc  w95             ; device belongs to table
        rjmp  w96             ; not match, goto "non Paged" Mode
w95:                          ; Entry Point for AVRProg Block Flash write
        ldi   s_data,0x40     ; write low byte
        rcall spi123
        ld    s_data, X+
        rcall wrser           ; wrser(s_data) SPI write (byte 4)
        ldi   s_data,0x48     ; write high byte
        mov   pol_cmd,s_data  ; save command for polling
        rcall spi123
        ld    s_data, X+
        mov   p_data,s_data  ; save data for polling
        mov   pol_al,addrl   ; save address for polling
        mov   pol_ah,addrh
        rcall wrser          ; wrser(u_data) SPI write (byte 4)
        adiw  addrl,1        ; Auto increment address
        dec   Bcnt2
        breq  w95b
        dec   Bcnt2
        breq  w95b
        tst   B_Mode
        breq  w95            ; Loop for "Block Mode Commands" see Note 16
        dec   Bcnt3          ; count Bytes for Pagesize
        brne  w95
w95b:
        tst   B_Mode
        brne  w95c
        rjmp  put_ret
w95c:
        dec   Bcnt3          ; Bcnt3 will be set correct after write
        ldi   s_data,0x4c    ; write Memory Page
        rcall wrser          ; wrser(0x4c)  SPI write (byte 1)
        mov   s_data,pol_ah  ; last written Adress is in Page to be programmed!
        rcall wrser          ; wrser(addrh) SPI write (byte 2)
        mov   s_data,pol_al  ; last written Adress is in Page to be programmed!
        rcall wrser          ; wrser(addrl) SPI write (byte 3)
        ldi   s_data,0x00
        rcall wrser          ; wrser(0x00)  SPI write (byte 4)
        mov   Bcnt3,Pagewords; reload Counter für Pagesize
        rjmp  wait_M
       
w96:                         ; Non Page Mode Flash write
        ld    p_data, X+     ; load byte to Poll data
        cpi   p_data,0xFF    ; if (p_data == 0xFF)
        breq  w961           ; skip burning
        ldi   s_data,0x40    ; write low byte
        mov   pol_cmd,s_data ; save command for polling
        rcall spi123
        mov   s_data,p_data  ; reload data from Poll data
        rcall wrser          ; wrser(s_data) SPI write (byte 4)
        mov   pol_al,addrl   ; save address for polling
        mov   pol_ah,addrh
        rcall Bws_pol
w961:
        dec   Bcnt2
        ld    p_data, X+     ; load byte to Poll data
        cpi   p_data,0xFF    ; if (p_data == 0xFF)
        breq  w962
        ldi   s_data,0x48    ; write high byte
        mov   pol_cmd,s_data ; save command for polling
        rcall spi123
        mov   s_data,p_data  ; reload data from Poll data
        rcall wrser          ; wrser(u_data) SPI write (byte 4)
        mov   pol_al,addrl   ; save address for polling
        mov   pol_ah,addrh
        rcall Bws_pol
w962:
        adiw  addrl,1        ; Auto increment address
        dec   Bcnt2
        brne  w96
        rjmp  put_ret        ; reply

;====== 'R' - Read Program Memory ==========================================
w10:
        cpi   u_data,'R'     ; 'R' Read program memory
        brne  w10B
        ldi   s_data,0x28    ; read high Byte (order is different from Block Read!)
        rcall wrser          ; wrser(0x28)  SPI write (byte 1)
        mov   s_data,addrh
        rcall wrser          ; wrser(addrh) SPI write (byte 2)
        mov   s_data,addrl
        rcall wrser          ; wrser(addrl) SPI write (byte 3)
        rcall rdser          ; SPI read  (byte 4)
        mov   u_data,s_data
        rcall putc           ; send data
        ldi   s_data,0x20    ; read Low Byte
        rcall wrser          ; wrser(0x20)  SPI write (byte 1)
        mov   s_data,addrh
        rcall wrser          ; wrser(addrh) SPI write (byte 2)
        mov   s_data,addrl
        rcall wrser          ; wrser(addrl) SPI write (byte 3)
        rcall rdser          ; SPI read  (byte 4)
        mov   u_data,s_data
        rcall putc           ; send data
        adiw  addrl,1        ; Auto increment address
        rjmp  waitcmd        ; goto waitcmd();

;====== 'g' - Block Read Memory =========================================
w10B:
        cpi   u_data,'g'     ; 'g' Block Read Program Memory
        brne  w11
        rcall getc           ; XH = getc();
        mov   XH,u_data
        rcall getc           ; XL = getc();
        mov   XL,u_data
        rcall getc           ; getc(Memorytype);
        cpi   u_data,'F'
        breq  w10B2
        cpi   u_data,'E'
        breq  w10B1
        rjmp  put_err
w10B1:
        rcall read_send_datamem
        sbiw  XL,1
        brne  w10B1
        rjmp  waitcmd        ; goto waitcmd();
w10B2:
        rcall read_send_progmem
        sbiw  XL,2
        brne  w10B2
        rjmp  waitcmd

;====== 'A' - Load Address =================================================
w11:
        cpi   u_data,'A'     ; 'A' Load address
        brne  w12
        rcall getc           ; addrh = getc();
        mov   addrh,u_data
        rcall getc           ; addrl = getc();
        mov   addrl,u_data
        rjmp  put_ret        ; goto reply();
       
;====== 'D' - Write Data Memory ============================================
w12:
        cpi   u_data,'D'     ; 'D' Write data memory
        brne  w13
        rcall getc           ; get data
        rcall eeprom_write
        rjmp  put_ret

;====== 'd' - Read Data Memory =============================================
w13:
        cpi   u_data,'d'     ; 'd' Read data memory
        brne  w14
        rcall read_send_datamem
        rjmp  waitcmd        ; goto waitcmd();

;====== 'L' - Leave Programming Mode =======================================
w14:
        cpi   u_data,'L'     ; 'L' Leave programming mode
        brne  w142
        sbi   PORTB,RESET    ; RESET = 1
        ldi   temp1,10       ; delay 10mS;
        rcall delay
        cbi   PORTB,RESET    ; RESET = 0
        rjmp  put_ret

;====== 'E' - Exit , release all Ports, inhibit AVR910 =====================
w142:
        cpi   u_data,'E'  ; 'E' xit
        brne  w15
        sbi   PORTB,RESET    ; RESET = 1
        ldi   temp1,10       ; delay 10mS;
        rcall delay
        cbi   PORTB,RESET    ; RESET = 0
        wdr
        ldi   temp1,24
        ldi   temp2,8
        out   WDTCSR,temp1   ;  WDT
        nop
        out   WDTCSR,temp2
        rjmp  put_ret        ; exit command for AVR Prog
 
;====== 'e' - Chip Erase ===================================================
w15:
        cpi   u_data,'e'     ; 'e' Chip erase
        brne  w16
        ldi   s_data,0xac
        rcall wrser          ; wrser(0xac) SPI write (byte 1)
        ldi   s_data,0x80
        rcall wrser          ; wrser(0x80) SPI write (byte 2)
        ldi   s_data,0x04
        rcall wrser          ; wrser(0x04) SPI write (byte 3)
        ldi   s_data,0x00
        rcall wrser          ; wrser(0x00) SPI write (byte 4)
        ldi   temp1,40       ; delay 40mS
        rcall delay
        rjmp  put_ret

;====== 'l' - Write Lockbits ==============================================

; writes Lockbits in SPI Byte 2 for old "classic" AVR Chips, and in addition
; writes Lockbits in SPI byte 4 for new Mega and Tiny Devices. 
; One Type does not care about the other Byte...
w16:
        cpi   u_data,'l'     ; 'l' Write Lockbits
        brne  w161
        rcall getc           ; get data
        ldi   s_data,0xac
        rcall wrser          ; wrser(0xac) SPI write (byte 1)
        mov   s_data,u_data
        ori   s_data,0xe0
        rcall wrser          ; wrser(u_data) SPI write (byte 2)
        rjmp  w162
       
;====== 'f' - Write Fusebits ==============================================
w161:
        cpi   u_data,'f'     ; 'f' Write Fusebits
        brne  w17
        rcall getc           ; get data
        ldi   s_data,0xac
        rcall wrser          ; wrser(0xac) SPI write (byte 1)
        ldi   s_data,0xa0
        rcall wrser          ; wrser(0xa0) SPI write (byte 2)
w162:
        clr   s_data
        rcall wrser          ; wrser(0x00)   SPI write (byte 3)
        mov   s_data,u_data
        rcall wrser          ; wrser(u_data) SPI write (byte 4)
        ldi   temp1,10       ; delay 10mS
        rcall delay
        rjmp  put_ret

;====== 's' - Read Signature Bytes =========================================
w17:
        cpi   u_data,'s'     ; 's' Read signature bytes
        brne  w18
        ldi   temp3,0x30
        mov   cmd1,temp3
        clr   cmd2
        ldi   temp3,0x02
        mov   cmd3,temp3
        rcall w123r4
        dec   cmd3
        rcall w123r4
        clr   cmd3
        rcall w123r4
        rjmp  waitcmd

;====== 'm' - Write Program Memory Page ====================================
w18:
        cpi   u_data,'m'     ; 'm' Write Program Memory Page
        brne  w19
        ldi   s_data,0x4c
        rcall wrser          ; wrser(0x4c)  SPI write (byte 1)
        mov   s_data,pol_ah  ; for speeding up transfer
        rcall wrser          ; wrser(addrh) SPI write (byte 2)
        mov   s_data,pol_al  ; for speeding up transfer
        rcall wrser          ; wrser(addrl) SPI write (byte 3)
        ldi   s_data,0x00
        rcall wrser          ; wrser(0x00)  SPI write (byte 4)
        rjmp  wait_M         ; write FLASH delay
       
;====== ':' - Universal Command ============================================
w19:
        cpi   u_data,':'     ; ':' Universal Command
        brne  w20
        rcall getc           ; get data1
        mov   cmd1,u_data    ; cmd1 = data1
        rcall getc           ; get data2
        mov   cmd2,u_data    ; cmd2 = data2
        rcall getc           ; get data3
        mov   cmd3,u_data    ; cmd3 = data3
        rcall w123r4
        ldi   temp1,50       ; delay 50mS
        rcall delay
        rjmp  put_ret
       
;====== '.' - New Universal Command ========================================
w20:
        cpi   u_data,'.'     ; '.' New Universal Command
        brne  w21
        rcall getc           ; get data1
        mov   cmd1,u_data    ; cmd1 = data1
        rcall getc           ; get data2
        mov   cmd2,u_data    ; cmd2 = data2
        rcall getc           ; get data3
        mov   cmd3,u_data    ; cmd3 = data3
        rcall getc           ; get data4
        rcall w1234
        mov   u_data,rd_s_data
        rcall putc           ; send data
        ldi   temp1,50       ; delay 50mS
        rcall delay
        rjmp  put_ret
 
;====== 'r' - Read Lock Bits ==============================================
w21:
        cpi   u_data,'r'     ; 'r' Read Lockbits
        brne  w22
        ldi   s_data,0x58    ; SPI load (byte 1)
        rjmp  w22a
       
;====== 'F' - Read Fusebits (lfuse) =========================================
w22:
        cpi   u_data,'F'     ; 'F' Read Fusebits
        brne  w23
        ldi   s_data,0x50    ; SPI load (byte 1)
w22a:
        rcall wrser          ; wrser(0x5n) SPI write (byte 1)
        clr   s_data         ; SPI load (byte 2)
        rjmp  w26

;====== 'N' - Read high Fusebits ==========================================
w23:
        cpi   u_data,'N'     ; 'N' Read high Fusebits
        brne  w24
        ldi   s_data,0x58
        rcall wrser          ; wrser(0x58) SPI write (byte 1)
        rjmp  w25
       
;====== 'Q' - Read extendet Fusebits ==============================================
w24:
        cpi   u_data,'Q'     ; 'Q' Read extendet Fusebits
        brne  w30
        ldi   s_data,0x50    ; SPI load (byte 1)
        rcall wrser          ; wrser(0x50) SPI write (byte 1)
w25:
        ldi   s_data,0x08    ; SPI load (byte 2)
w26:
        rcall wrser          ; wrser(0x00) SPI write (byte 2)
        clr   s_data
        rcall wrser          ; wrser(0x00) SPI write (byte 3)
        rcall rdser          ;             SPI read  (byte 4)
        mov   u_data,s_data
        rcall putc           ; send data
        rjmp  waitcmd

;====== Unknown Command ====================================================
w30:    rjmp  put_err

;====== Wait for FLASH write in avr910 mode ==================================
wait_S:
    table Dev_M
        rcall bel_table
        brcs  ws_pol        ; No Page Mode, poll last Byte
        rjmp  put_ret       ; in Byte Mode time is long enougt to get next Byte
Bws_pol:                    ; entry adress for Block mode delay
        ldi   temp3,1
        mov   B_Flag,temp3  ; set Flag that ws_poll comes back with ret ...
ws_pol:
        tst   PollcodeF     ; if polling not applicable, standart delay
        breq  ws_del        ; polling not used
ws_def:
        cp    p_data,PollcodeF ; if (data == PollcodeF)
        breq  ws_del           ; wait default delay
        andi  pol_cmd,0x0f     ; write command: 0x48, 0x40
        ori   pol_cmd,0x20     ; read  command: 0x28, 0x20
        clr   temp3            ; clear polling counter
ws_cy:
        mov   s_data,pol_cmd
        rcall wrser            ; wrser(pol_cmd) SPI write (byte 1)
        mov   s_data,pol_ah
        rcall wrser            ; wrser(pol_ah) SPI write (byte 2) (S89=byte1)
        mov   s_data,pol_al
        rcall wrser            ; wrser(pol_al) SPI write (byte 3) (S89=byte2)
        rcall rdser            ;               SPI read  (byte 4) (S89=byte3)
        cp    s_data,p_data
        breq  ws_ok            ; s_data = p_data
        dec   temp3
        brne  ws_cy            ; loop
ws_del:         ; 256 polling cycles are over, give additional standart Time
        ldi   temp1,10         ; delay 10mS
        rcall delay
ws_ok:
        tst   B_Flag
        breq  put_ret
        clr   B_Flag           ; Reset B_Flag for normal operation
        ret
       
;====== Wait for FLASH write in page mode ==================================
wait_M:
        cpi   p_data, 0xFF     ; if last Byte was 0xFF give standart delay
        breq  wm_del
        andi  pol_cmd,0x0f     ; write command: 0x48, 0x40
        ori   pol_cmd,0x20     ; read  command: 0x28, 0x20
        clr   temp3            ; clear polling counter
wm_cy:
        mov   s_data,pol_cmd
        rcall wrser            ; wrser(pol_cmd) SPI write (byte 1)
        mov   s_data,pol_ah
        rcall wrser            ; wrser(pol_ah) SPI write (byte 2)
        mov   s_data,pol_al
        rcall wrser            ; wrser(pol_al) SPI write (byte 3)
        rcall rdser            ;               SPI read  (byte 4)
        cp    s_data,p_data
        breq  wm_ok            ; s_data = p_data
        dec   temp3
        breq  wm_del   ; 256 polling cycles are over, give another 50ms delay...
        rjmp  wm_cy            ; loop
wm_del:
        ldi   temp1,50         ; delay 50mS
        rcall delay
wm_ok:
        tst   B_Mode
        breq  wm_end
        tst   Bcnt2
        breq  wm_end
        rjmp  w95
wm_end:
        clr   B_Mode           ; Reset Block Mode Flag
        rjmp  put_ret

;====== Command Error ======================================================
put_debug:                     ; shows u_data on serial for debug reason
        rcall putc
put_err:
        ldi   u_data,'?'       ; putc('?')
        rcall putc             ; send '?'
        rjmp  waitcmd
 
;====== Reply Command ======================================================
put_ret:
        ldi   u_data,0x0d      ; putc(0x0D)
        rcall putc             ; send CR
        rjmp  waitcmd

;***************************************************************************

;======   =====================================================
;  Vcc
VccON:     ldi   temp1,00
           out   OCR1AH,temp1
           out   OCR1AL,temp1
           out   TCNT1H,temp1
           out   TCNT1L,temp1
           ldi   temp1,$7C       ; 0111.1100
           out   PORTD,temp1
           out   DDRD,temp1      ; PD2...6 - OUT=1
           ldi   temp1,$40
           out   TCCR1A,temp1    ; TCNT1- CTC
           ldi   temp1,$09
           sbi   DDRB,3          ; PB3 - OUT
           out   TCCR1B,temp1    ; CLK=1/1  TCNT1
           rcall WaitOFF         ;     
;  Vcc
           ldi   temp1,00
           out   TCCR1B,temp1    ;  TCNT1
           out   TCCR1A,temp1
           cbi   PORTB,3
           cbi   DDRB,3          ; PB3 - IN
           out   PORTD,temp1
           out   DDRD,temp1      ; PORTD - IN
           rcall WaitOFF         ;     
           rjmp  VccON

; /     
WaitOFF:   wdr
           clr   temp1
WaitOFF0:  sbis  PINB,0          ;   
           rjmp  WaitOFF         ; 
           rcall Zad
           dec   temp1
           brne  WaitOFF0        ; 
WaitON:    wdr
           clr   temp1
WaitON0:   sbic  PINB,0          ;  
           rjmp  WaitON          ;  
           rcall Zad
           dec   temp1
           brne  WaitON0         ; 
           ret

; / 
Zad:       clr   temp2
Zad11:     dec   temp2
           brne  Zad11
           ret

;***************************************************************************
;* TABLE
;* device codes
;***************************************************************************
; byte write
; avr910-devcode, Code during polling Flash.
Dev_S:
       .db  0x68 ,0xFF       ;AT90S8535
       .dw  0                ;End of table
 
; Devices which support Page Programming. Maximum ist 0x40 due to limitated
; RAM in 2313 Chip (Pages with bigger Pages are programmed multiple times).
Dev_M:
       .db  0x69 ,0x20       ;ATmega8535
       .dw  0                ;End of Table

;***************************************************************************
;* TABLE
;* revision codes
;***************************************************************************
SW_Ver:
 .db "02",0,0
HW_Ver:
 .db "03",0,0

;***************************************************************************
;* TABLE
;* ID string "AVR ISP"
;***************************************************************************
ID_Str:
 .db "AVR ISP",0

